/* create a "description cache" for use by the SRCHINDX add-on for
SRE-http */

/* Note:
 by default, files are assumed to be non-html text files. 
 Exceptions:
    Files with extensions in the Htmls list are assumed to be HTML documents
    Files with extensions that appear in the NoContents variable 
      (either the user-set NoContents variable, or the NoContents entry
       in the  Swish Configuration file) are assumed to be non-text files.
      (descriptions are not generated for non-text files)
*/


/* -------- User Changable Parameters   -------------*/


/* Files with these extensions are assumed to be HTML files */
htmls=" HTM HTML SHTML SHTM HTML-SSI HTM-SSI HM "

/* Files with these extensions are assumed to NOT  be plain-text files --
THIS IS ONLY USED if you are using the "list of URLS" option --
if you are reading from a SWISH index, the NOCONTENTS parameter is used */
nocontents="JPG GIF ZIP XBM "

/* These are used as default "summaries" */
gmess.1=' File not available'
gmess.2=' Summary not available'
gmess.3=' Explicit summary not available'
gmess.4=' Error in routine -- no summary available'


/* Directory specific "description files". These should contain
 descriptions of files within the directory. */
descript_file="DESCRIBE.TXT"


/* the default SWISH configuration file */
defcon="SAMPLES.CON"

/* the default list of urls (text mode) */
deftxt="SAMPLES.IN"

/* the default "description-cache" file */
defdesc="SAMPLES.DCH"

/* the default "GoServe data directory */
defdir="\WWW"

/* the default "directory specific file-description file" */
defdescribe='DESCRIBE.TXT'


/* This is the character used to signal "continuation of a description"
I.e. (assuming continuation_flag='|'
FOOBAR.TXT   This is the descripton of foobar.txt
 |           And this is the second line.
Note that the | should be the first non space character */
continuation_flag='|'

/* -------- End of User Changable Parameters   -------------*/

crlf='0d0a'x

call initit

say "      "cy_ye ' This is the SRE-http "description cache creator". ' normal
say " It is meant to be used in conjunction with the SRCHINDX add-on, "
say " and the SWISH indexer.  "
say " "
say " Typically, you will need to first create a SWISH index file by "
say " running SWISH version 1.1 (see SRCHINDX.DOC for details)."
say reverse "  Newer/older versions of Swish (other then 1.1) may break this program " normal
say " "
say " Alternatively, if you only intend to allow searches of filenames and "
say ' summaries, you can provide an explicit "list of URLs" instead.'
say " "

aa=yesno(' Create (C), or modify (M), a description cache',,'CREATE MODIFY')

if aa=0 then do
   call editit
   say " bye "
   exit
end  /* Do */

aa=yesno(" Use a SWISH index (S), or a text index (T) ",,'SWISH TEXT')

if aa=1 then
  call get_confile
else
  call get_txtfile

call get_filelist_info
call get_outname

say reverse "  ------------------------------ " normal
say " Saving descriptions for " filelist.0 " files "
latesd.!dir=' '
do m=1 to filelist.0
   bigd.m=translate(filelist.m.!original,'/','\')
   bigd.m.!title=filelist.m.!title
   bigd.m.!size=filelist.m.!size

   bigd.m.!YES=strip(make_summary(filelist.m,filelist.m.!type,1))
   if yaman=2 then  
       bigd.m.!create=bigd.m.!yes
   else
       bigd.m.!create=strip(make_summary(filelist.m,filelist.m.!type,2))

   bigd.m.!gotit=yaman  /*0-none, 1=created, 2=explicit */

  if (m//100)=1 then say " Processed " m " files of " filelist.0
end /* do */
bigd.0=filelist.0
a=cvwrite(outname,bigd)
say " "
say bold " Description-cache index file ("outname") has been created." normal
exit


/**************************/
get_outname:
say reverse "  ------------------------------ " normal
n2:

say " Enter the (fully qualifed) name to use for the description index-file  "
say " (it should be relative to your GoServe working directory)"
call charout ,' (default= ' defdesc ')' bold '  ? ' normal
pull outname ; if outname="" then outname=defdesc

outname=strip(outname)

adir=filespec('D',outname)||filespec('P',outname)
if adir="" then adir=directory()
if dosisdir(strip(adir,'t','\'))=0 then do
   say "    Could not find directory: " reverse adir normal
   say "    Please re-enter .... "
   signal n2
end

n4a:
say " "
say ' Enter the  name  of the "directory specific" 'bold' file-description file. 'normal
call charout ,' (default= ' defdescribe ', ?=HELP, .=None)' bold '  ? ' normal
pull fdescribe ; if fdescribe="" then fdescribe=defdescribe
else
if fdescribe="?" then do
   say ' The "directory specific"' bold' file-description file'normal' is used to assign '
   say " explicit descriptions to any file.  "
   say ' For all files being "described", a 'bold' file-description file'normal " in it's " bold"own"normal
   say " directory is examined; and if a matching entry is found,  the associated"
   say " description is used."
   say " Entries in the file-description file should be organized as:"
   say "     FILE1.xxx a description "
   say " Examples: "
   say cy_ye "     " normal " file2.yyy  This is the YYY file "
   say cy_ye "     " normal " foobar.htm  This is the classic FOOBAR file. In this case we use"
   say "          | a 2 line description (the | is  a continuation flag)"
   signal n4a
end  /* Do */
fdescribe=translate(fdescribe,'\','/')
if pos('\',fdescribe)>0 then do
   say " The file-description file is " bold" directory-specific "normal
   say " Please reenter (and do NOT include a path) "
   signal n4a
end


return 0

/**************************/
/* read swish configuration and index files */
get_confile:
say reverse "  ------------------------------ " normal
n2a:

say " Enter the fully qualifed name of the reference SWISH configuration file."
call charout , ' (default=' defcon ')' bold '   ? ' normal
pull confile
if confile="" then confile=defcon

/* now get info */
foo=fileread(confile,clines,,'E')
if clines.0=0 then do
   say "    Could not find configuration file: " reverse confile normal
   say "    Please re-enter .... "
   signal n2a
end

/* find the IndexFile entry, and the ReplaceRules entries. */
nreps=0 ; nocontents=' '
do mm=1 to clines.0
   aline=strip(upper(clines.mm))
   select  
      when  abbrev(aline,'INDEXFILE')=1 then do
          parse var aline . daindx . ; daindx=strip(daindx)
      end
      when abbrev(aline,'REPLACERULES') then do
          nreps=nreps+1 ; aline=translate(aline,' ','"'||"'")
          parse var aline  . . reprules.nreps.!orig reprules.nreps.!new .
      end
      when abbrev(aline,'NOCONTENTS')=1 then do
          parse var aline . nocontents . ; nocontents=strip(nocontents)
      end
      otherwise nop
    end
end  /* Do */

foo=daindx
daindx=stream(daindx,'c','query exists')
if isswi=" " then do
   say " Problem: could not find SWISH index file: " foo
   say " (you may need to run this program from a different directory) "
   exit
end
do mm=1 to nreps
   a1=reprules.mm.!Orig ; a2=reprules.mm.!new
   reprules.mm.!orig=strip(upper(a1))
   reprules.mm.!new=strip(upper(a2))
end /* do */

/* set some stuff fron daindx */
say " Processing " daindx 
call linein daindx,1,0
do mm=1 to 3
   foo=linein(daindx)
end /* do */
wow=linein(daindx)
parse var wow . ',' nfiles .; nfiles=strip(nfiles)

/* count lines in the file */
call linein daindx,1,0
ndo=0
do until lines(daindx)=0
   foo=linein(daindx)
   ndo=ndo+1
end /* do */
say "     Extracting file names ... "
/* now get the lines ndo-nfiles to ndo-1 */
call linein daindx,1,0
do ij=1 to ndo-(1+nfiles)
   foo=linein(daindx)
end /* do */

/* convert url style names back to original files */
do nf=1 to nfiles
   baa=linein(daindx)
   parse var baa aa  '"' atitle '"' asize .
   afil=upper(strip(word(aa,1)))
   filelist.nf.!original=afil
   filelist.nf.!title=atitle
   filelist.nf.!size=asize
   do il=1 to nreps             /* convert to fully qualified names */
       if abbrev(afil,reprules.il.!new)=1 then do
             aa=reprules.il.!orig||substr(afil,1+length(reprules.il.!new))
             leave
       end  /* Do */
   end /* do */
   filelist.nf=aa
 
end /* do */
filelist.0=nfiles
return nfiles


/**************************/
/* read text (user created) configuration and index files */
get_txtfile:
say reverse "  ------------------------------ " normal
n3a:

say " Enter a text file containing a list of URLs (? for HELP) "
call charout , ' (default=' deftxt ')' bold '   ? ' normal
pull txtfile
if txtfile="" then txtfile=deftxt
if  txtfile="?" then do
  say "    Each line of the file should contain entries of the form: "
  say cy_ye "  " normal ' relative_url  "Short Description "  size filename '
  say " Where: "
  say "  " bold " relative_url "normal "is required. It is used as the link to the file. "
  say "  " bold ' "short description" ' normal ' is optional. If included, it must be within "'
  say "  " bold " size  "normal" is optional; it's the size in bytes "
  say "   " bold  "filename" normal 'is optional. It is the fully qualified name of the file.'
  say "       If not specified,  the URL is assumed to refer to a file that is "
  say "       relative to the GoServe data directory "
  say bold "Examples: " normal
  say cy_ye "  " normal '/samples/SAMPMBOX.HTM "Sample Message Sender for SRE-http" 1390 '
  say cy_ye "  " normal '/samples/SAMPOPT1.HTM "Sample of OPTIONS Keyphrase for SRE-http" 2728'
  say " "
  signal n3a
end

/* now get info */
foo=fileread(txtfile,clines,,'E')
if clines.0=0 then do
   say "    Could not find list of URLS: " reverse txtfile normal
   say "    Please re-enter .... "
   signal n3a
end


n3b:
say " Enter the name of the GoServe data directory: "
call charout , ' (default=' defdir ')' bold '   ? ' normal
pull datadir
if datadir=""  then datadir=defdir
datadir=strip(datadir,'t','\')
if dosisdir(datadir)=0 then do
  say " Could not find directory: " datadir
  signal n3b
end
/* process file list */
Say " Processing " txtfile
nfiles=0
do nf=1 to clines.0
   baa=clines.nf
   if baa=" " | abbrev(strip(baa),';')=1 then iterate
   nfiles=nfiles+1
   parse var baa aa  '"' atitle '"' asize absfile .
   afil=upper(strip(word(aa,1)))
   filelist.nfiles.!original=afil
   filelist.nfiles.!title=atitle
   filelist.nfiles.!size=asize

   if absfile <> " " then 
      filelist.nfiles=absfile
   else
       filelist.nfiles=datadir||strip(aa,'l','\')
   filelist.nfiles=translate(filelist.nfiles,'\','/')
   if filelist.nfiles.!size=" " | datatype(filelist.nfiles.!size)<>"NUM" then do
      filelist.nfiles.!size=dosdir(filelist.nfiles,'S')
  end


end /* do */
filelist.0=nfiles
return nfiles




/****************************/
/* given a filefilst, get descriptions */
get_filelist_info:
/* determine type of file: 2=text, 1=html, 0=non-text */
htmls=translate(upper(htmls),' ','.')
nocontents=translate(upper(nocontents),' ','.')

do mm=1 to filelist.0
   aff=filelist.mm
   filelist.mm.!type=2               /* assume it's text */
   foo=lastpos('.',aff)
   if foo=0 then iterate

   anext=strip(upper(substr(aff,foo+1)))
   if wordpos(anext,htmls)>0 then do
        filelist.mm.!type=1
        iterate
   end
   if wordpos(anext,nocontents)>0 then filelist.mm.!type=0
 
end /* do */

return 0



/*******************************************/
/* some initializations */
initit:     
/*---- load the rexxlib library */
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 droprxlib=1 ; gotrxlib=0
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
    say " Could not find the REXXLIB procedure library (REXXLIB.DLL). "
    say "  Did you download it? "
    exit
end  /* Do */

ansion=checkansi()
if ansion=1 then do
  aesc='1B'x
  cy_ye=aesc||'[37;46;m'
  normal=aesc||'[0;m'
  bold=aesc||'[1;m'
  re_wh=aesc||'[31;47;m'
  reverse=aesc||'[7;m'
end
else do
  say " Warning: Could not detect ANSI....  everything will look ugly ! "
  cy_ye="" ; normal="" ; bold="" ;re_wh="" ;
  reverse=""
end  /* Do */

return 0


 /* ------------------------------------------------------------------ */
 /* function: Check if ANSI is activated                               */
 /*                                                                    */
 /* call:     CheckAnsi                                                */
 /*                                                                    */
 /* where:    -                                                        */
 /*                                                                    */
 /* returns:  1 - ANSI support detected                                */
 /*           0 - no ANSI support available                            */
 /*          -1 - error detecting ansi                                 */
 /*                                                                    */
 /* note:     Tested with the German and the US version of OS/2 3.0    */
 /*                                                                    */
 /*                                                                    */
 CheckAnsi: PROCEDURE
   thisRC = -1

   trace off
                         /* install a local error handler              */
   SIGNAL ON ERROR Name InitAnsiEnd

   "@ANSI 2>NUL | rxqueue 2>NUL"

   thisRC = 0

   do while queued() <> 0
     queueLine = lineIN( "QUEUE:" )
     if pos( " on.", queueLine ) <> 0 | ,                       /* USA */
        pos( " (ON).", queueLine ) <> 0 then                    /* GER */
       thisRC = 1
   end /* do while queued() <> 0 */

 InitAnsiEnd:
 signal off error
 RETURN thisRC


/***************/
/* ------------------------------------- */
/* get summary info, or get explicit description in fdescribe (DESCRIBE.TXT)
file 
afilename: fully qualified filename to investigate
atype: 1- html, 2-non-html text, 0-non text 
asummary: 1- explicit, 2-create

returns a text or html summary, or a numeric code:
1= File not available
2= Summary not available
3= Explicit summary not available
4= Error in routine -- no summary available

yaman is also returned:
 0-no description, 1=created, 2=explicit 
 

 */

make_summary:procedure  expose yaman atitle asize fdescribe latestd. continuation_flag
parse arg afilename,atype,asummary

yaman=0
eek=stream(afilename,'c','query exists')   /* check for existence*/
if eek="" then return 1            /* error */

/* check for explicit description file */
if fdescribe<>" " then do
 checkd=strip(upper(strip(filespec('d',afilename)||filespec('p',afilename),'t','\')'\'))
 if checkd<> latestd.!dir then do 
    call make_desc(checkd)      /* saves to latestd. */
    latestd.!dir=checkd
 end

 fnm=strip(upper(filespec('n',afilename)))   /* check the descriptions, and return match if found */

 foo=arraysearch(latestd,res1,fnm,'X')
 if res1.0>0 then do
       iif=res1.1
       yaman=2
       return latestd.iif.!desc
  end
end             /* check description file */
 
/* no explicit description -- perhaps create one? */
select

  when atype=0 then            /* not text, so no description */
      return 2
 
  when atype=2 & asummary<>2 then   /* non-html text, non-create mode */
      return 2


  when atype=2 & asummary=2 then do   /* non-html text, create mode */
       alen=min(chars(afilename),300)
       stuff=charin(afilename,1,alen)
       fpp=stream(afilename,'c','close')
       yaman=1
     wow=replacestrg(wow,'&','&amp;','ALL')
     wow=replacestrg(stuff,'<','&lt;','ALL')
     wow=replacestrg(wow,'>','&gt;','ALL')
     wow=replacestrg(wow,'"','&quot;','ALL')

       return wow
  end

  when atype=1 then do
     alen=min(chars(afilename),10000)
     stuff=charin(afilename,1,alen)
     fpp=stream(afilename,'c','close')
     stuff=space(translate(stuff,' ','00090a0d1a1b'x))
     wow=look_header(afilename)
     if wow<>0 then do
         yaman=2
        return wow
     end  /* Do */

    if wow=0 & asummary<>2 then 
       return 3

    WOW=LOOK_HTAG()                     /* use <Hn> for summary */
    if wow<>0 then do 
       yaman=1
       return wow
    end  /* Do */
    return 3
  end
 
  otherwise do
     say " ERROR: should not be here in make summary "
     return 4
  end
end

/******************/
/* read a description file with possible continuation lines */
make_desc:
parse arg dafile
 foo2=checkd||fdescribe
 foo=fileread(foo2,tmps,,'E')
 igoo=0
 daflag=left(strip(continuation_flag),1)

/* continuation lines have daflag (typically |) as first non-space character */
ison=0 ; igoo=0
do mm=1 to tmps.0
   al=strip(tmps.mm)
   if ison=1 &   abbrev(al,daflag)=1  & daflag<>" " then do    /* this is acontinuation */
            parse var al (daflag) eek
            intos.igoo=intos.igoo||' '||eek
   end  /* Do */
  if al=" " | abbrev(al,';')=1 then do 
      ison=0 ; iterate
  end  /* Do */
  igoo=igoo+1
  intos.igoo=al
  ison=1   
end
intos.0=igoo

igoo=0
do ji=1 to intos.0
   igoo=igoo+1
   parse var intos.ji latestd.igoo latestd.igoo.!desc
   latestd.igoo=strip(upper(latestd.igoo))
end             /* refresh description file */
latestd.0=igoo

if igoo>0 then say igoo " entries in file-description file: " foo2
return igoo



/* ----------------------------------------------------------------------- */
/* Look for "desc" field in header     */
/* ----------------------------------------------------------------------- */

look_header: procedure expose stuff url_title
parse arg afile
dowrite=0
do until stuff=""

    parse var stuff  p1 '<' tag '>' stuff
    if  translate(word(tag,1))="HEAD" then do   /* now in head !*/
            dowrite=1
            iterate
    end
    if dowrite=0 then iterate    /* wait till we get into head .. */

    if  translate(word(tag,1))="/HEAD" then  /* out of head, all done ! */
        leave

/* IT IS A TITLE TAG?  */
     if translate(word(tag,1))="TITLE" then do
        parse var stuff url_title '<' footag '>' stuff
     end

/* is it a  META HTTP-EQUIV or a META NAME ? */
    if translate(word(tag,1))="META" then do
        parse var tag ameta atype '=' rest
        tatype=translate(atype)
        if tatype="HTTP-EQUIV" | tatype="NAME" then do
           parse var rest aval1 rest
           REST=STRIP(REST)

           aval1=strip(aval1) ;
           aval1=strip(aval1,,'"')
           if abbrev(translate(aval1),'DESC')<>1 then iterate

           aval2=" "
           foo1=ABBREV(translate(rest),'CONTENT')
           if foo1>0 then do
                PARSE VAR REST FOO '=' AVAL2
                aval2=strip(aval2)
                aval2=strip(aval2,'b','"')
                WOW=LEFT(AVAL2,500)
                return WOW
           end
        end             /* name or http-equiv */
    end         /* meta */
end             /* stuff */

if stuff="" then say "Warning: </head> tag NOT found: " afile

return 0


/* ----------------------------------------------------------------------- */
/* Extract <hn> fields     */
/* ----------------------------------------------------------------------- */

look_htag: procedure expose stuff filename

stuff0=left(stuff,1000)

amessage=""
dowrite=0
do until stuff=""
    parse var stuff  p1 '<' tag '>' stuff
    ttag=translate(word(tag,1))
    if wordpos(ttag,' H1 H2 H3 H4 TITLE')>0 THEN DO   /* grab stuff */
        parse var stuff  amess '<' tag2 '>' stuff
        amessage=amessage||amess||'<b> | </b>'
    end
end

if amessage="" then do  /* getting desperate -- grab any old words! */
   do until stuff0=""
      parse var stuff0 p1 '<' tag '>' stuff0
      amessage=amessage||' '||p1
   end
end

if amessage="" then
   return 0
amessage=left(amessage,300)  /* keep it short */
return amessage




/* ------------- */
/* ----------------------------------------------------------------------- */
/* REPLACESTRG: In string astring, find first occurence substring target and
.   replace it with substring putme
.      if no target, return unchanged astring
.      if no putme, then remove target
.      if type=backward, then find/change LAST occurence
.      if type=all, find/change all occurences
.      if exactmatch=yes, then do not capitalize during search (exact match only */
/* ----------------------------------------------------------------------- */

replacestrg: procedure

exactmatch=0
backward=0 ; doall=0

parse arg astring ,  target   , putme , type , exactmatch

type = translate(type)
if type="BACKWARD" then backward="YES"
if type="ALL" then doall="YES"

iat=1
joelen=length(target)
joelen2=length(putme)

doagain:                /* here if doall=yes */
 if exactmatch="YES" then do
    if   backward="YES" then
        joe= lastpos(target,astring)
    else
        joe= pos(target,astring,iat)
 end
 else do
   if   backward="YES" then
        joe= lastpos(translate(target),translate(astring))
    else
        joe= pos(translate(target),translate(astring),iat)
 end
 if joe=0 then
         return astring

 astring=delstr(astring,joe,joelen)
 if putme<>' ' then
    astring=insert(putme,astring,joe-1)

 if doall="YES" then do
     iat=joe+joelen2
     signal doagain
 end
/* else, all done */
 return astring

/* -------------------- */
/*********************************/
/* rudimentary edit of a description file */

editit:
say " "
say reverse "  ------------------------------ " normal
iff=1

n2b:
say " Enter the name of the description-cache file you want to modify."
call charout ,' (default= ' defdesc ')' bold '  ? ' normal
pull outname ; if outname="" then outname=defdesc

outname=strip(outname)

foo=cvread(outname,aindx)
if foo=0 then do
   say "    Could not find description cache file: " reverse outname normal
   say "    Please re-enter .... "
   signal n2b
end
say " "
say outname " has " aindx.0 " entries "

if yesno(' Would you like to list the names of these entries ')=1 then do
    SAY CY_YE " File Name    ::     Title  "
     do mm=1 to aindx.0
         say reverse strip(upper(aindx.mm)) normal bold ' :: ' normal  aindx.mm.!title
         if (mm//20)=0 then do
            call charout , cy_ye ' (hit any key to continue, X stop) ' normal
            foo=sysgetkey("noecho") ; say " "
            IF UPPER(FOO)='X' then LEAVE
         end  /* Do */
     end /* do */
end  /* Do */

do forever
    say   " Enter the " reverse " File Name " normal " you wish to modify: "
    thedef=aindx.iff
    say bold " File Name  (ESC to exit, ^C to abort)? " normal
    todo=strip(upper(stringin(,2,thedef,40))); say " "
    if todo=" " then leave
    eek=arraysearch(aindx,res1,todo,'X')
    if res1.0=0 then do
        say " No entry for: " todo ; say
        iterate
    end  /* Do */
    iff=res1.1
    say " "
    say cy_ye " .......................... " normal
    say reverse aindx.iff normal  " ( size= " aindx.iff.!size
    say bold "Title:" normal aindx.iff.!title
 
   dathing=aindx.iff.!yes
   if datatype(dathing)="NUM" then do   /* replace special codes */
        if dathing>0 & dathing<5 then dathing=gmess.dathing
   end
   dathing=fixda(dathing)
    say bold "1) Explicit summary:" normal 
    say dathing

   dathing=aindx.iff.!create
   if datatype(dathing)="NUM" then do   /* replace special codes */
        if dathing>0 & dathing<5 then dathing=gmess.dathing
    end
    dathing=fixda(dathing)

    say bold "2) Created summary: " normal
    say dathing

    say cy_ye " .......................... " normal
    call charout, reverse " Change: "  normal
    call charout, bold " 0" normal "=Nothing, "
    call charout, bold " 1" normal "=Explicit Summary,"
    call charout, bold " 2" normal "=Created Summary? "
    parse pull which
    if which=1 then do
       say " Enter new Explicit Summary: "
       dathing=getda()
       aindx.iff.!yes=dathing
   end
   if which=2 then do
       say " Enter new Created Summary: "
       dathing=getda()
       aindx.iff.!create=dathing
   end
end

say " Saving Changes to ... " outname
a=cvwrite(outname,aindx)
if a=1 then 
  say ".... changes sucessfully saved."
else
 say " PROBLEM saving changes "

return 0


getda:

say " Enter several lines of text, a blank lines signals end "
poo=""
do forever
   call charout ," ? "
      parse pull astuff
   if astuff<>"" then
      poo=poo||crlf||astuff
   else
      leave
end
return poo


fixda:procedure expose crlf
parse arg dathing

dathing=space(translate(dathing,' ','090a0d001a'x))
aa="" ; ict=0
foo=scrsize() ; 
parse var foo . mxsize
mxsize=mxsize-5
do mm=1 to words(dathing)

  aw=word(dathing,mm)
  if ict+length(aw)>mxsize then do
     aa=aa||crlf
     ict=0
   end
   aa=aa||' '||aw
   ict=ict+length(aw)+1
end
return aa

/* -------------------- */
/* choose between 2 alternatives (by default,a yes or no ), 
return 1 if yes (or the first alternative in the altans list) */
yesno:procedure expose normal reverse bold
parse arg fooa , allopt,altans
if altans<>" " & words(altans)>1 then do
   w1=strip(word(altans,1))
   w2=strip(word(altans,2))
   a1=left(w1,1) ; a2=left(w2,1)
   a1a=substr(w1,2) ; a2a=substr(w2,2)
end
else do
    a1='Y' ; a1a='es'
    a2='N' ; a2a='o'
end  /* Do */
ayn='  '||bold||a1||normal||a1a||'\'||bold||a2||normal||a2a
if allopt=1 then  ayn=ayn||'\'||bold||'A'||normal||'ll'

do forever
 foo1=normal||reverse||fooa||normal||ayn
 call charout,  foo1 normal ':'
 pull anans
 if abbrev(anans,a1)=1 then return 1
 if abbrev(anans,a2)=1 then return 0
 if allopt=1 & abbrev(anans,'A')=1 then return 2
end



